@@ -4,8 +4,9 @@ import datetime
 import sys
 import warnings
 import time as mod_time
-from redis._compat import (b, izip, imap, iteritems, iterkeys, itervalues,
-                           basestring, long, nativestr, urlparse, bytes)
+from redis._compat import (b, basestring, bytes, imap, iteritems, iterkeys,
+                           itervalues, izip, long, nativestr, urlparse,
+                           unicode)
 from redis.connection import ConnectionPool, UnixDomainSocketConnection
 from redis.exceptions import (
     ConnectionError,
@@ -109,19 +110,53 @@ def parse_info(response):
     return info
 
 
+SENTINEL_STATE_TYPES = {
+    'can-failover-its-master': int,
+    'info-refresh': int,
+    'last-hello-message': int,
+    'last-ok-ping-reply': int,
+    'last-ping-reply': int,
+    'master-link-down-time': int,
+    'master-port': int,
+    'num-other-sentinels': int,
+    'num-slaves': int,
+    'o-down-time': int,
+    'pending-commands': int,
+    'port': int,
+    'quorum': int,
+    's-down-time': int,
+    'slave-priority': int,
+}
+
+
+def parse_sentinel_state(item):
+    result = pairs_to_dict_typed(item, SENTINEL_STATE_TYPES)
+    flags = set(result['flags'].split(','))
+    for name, flag in (('is_master', 'master'), ('is_slave', 'slave'),
+                       ('is_sdown', 's_down'), ('is_odown', 'o_down'),
+                       ('is_sentinel', 'sentinel'),
+                       ('is_disconnected', 'disconnected'),
+                       ('is_master_down', 'master_down')):
+        result[name] = flag in flags
+    return result
+
+
 def parse_sentinel(response, **options):
     "Parse the result of Redis's SENTINEL command"
-    output = []
-    parse = options['parse']
-
+    parse = options.get('parse')
     if parse == 'SENTINEL_INFO':
-        for sub_list in response:
-            it = iter(sub_list)
-            output.append(dict(izip(it, it)))
-    else:
-        output = response
-
-    return output
+        return [parse_sentinel_state(item) for item in response]
+    elif parse == 'SENTINEL_INFO_MASTERS':
+        result = {}
+        for item in response:
+            state = parse_sentinel_state(item)
+            result[state['name']] = state
+        return result
+    elif parse == 'SENTINEL_ADDR_PORT':
+        if response is None:
+            return
+        return response[0], int(response[1])
+    return response
 
 
 def pairs_to_dict(response):
@@ -130,6 +165,16 @@ def pairs_to_dict(response):
     return dict(izip(it, it))
 
 
+def pairs_to_dict_typed(response, type_info):
+    it = iter(response)
+    result = {}
+    for key, value in izip(it, it):
+        if key in type_info:
+            value = type_info[key](value)
+        result[key] = value
+    return result
+
+
 def zset_score_pairs(response, **options):
     """
     If ``withscores`` is specified in the options, return the response as
@@ -196,6 +241,21 @@ def parse_script(response, **options):
     return response
 
 
+def parse_scan(response, **options):
+    return response
+
+
+def parse_hscan(response, **options):
+    cursor, r = response
+    return cursor, r and pairs_to_dict(r) or {}
+
+
+def parse_zscan(response, **options):
+    score_cast_func = options.get('score_cast_func', float)
+    it = iter(response[1])
+    return [response[0], list(izip(it, imap(score_cast_func, it)))]
+
+
 class StrictRedis(object):
     """
     Implementation of the Redis protocol.
@@ -260,7 +320,11 @@ class StrictRedis(object):
             'SCRIPT': parse_script,
             'SET': lambda r: r and nativestr(r) == 'OK',
             'TIME': lambda x: (int(x[0]), int(x[1])),
-            'SENTINEL': parse_sentinel
+            'SENTINEL': parse_sentinel,
+            'SCAN': parse_scan,
+            'SSCAN': parse_scan,
+            'HSCAN': parse_hscan,
+            'ZSCAN': parse_zscan
         }
     )
 
@@ -309,7 +373,7 @@ class StrictRedis(object):
                 'decode_responses': decode_responses,
             }
             # based on input, setup appropriate connection args
-            if unix_socket_path:
+            if unix_socket_path is not None:
                 kwargs.update({
                     'path': unix_socket_path,
                     'connection_class': UnixDomainSocketConnection
@@ -324,6 +388,9 @@ class StrictRedis(object):
 
         self.response_callbacks = self.__class__.RESPONSE_CALLBACKS.copy()
 
+    def __repr__(self):
+        return "%s<%s>" % (type(self).__name__, repr(self.connection_pool))
+
     def set_response_callback(self, command, callback):
         "Set a custom Response Callback"
         self.response_callbacks[command] = callback
@@ -511,6 +578,26 @@ class StrictRedis(object):
             parse = 'SENTINEL'
         return self.execute_command('SENTINEL', *args, **{'parse': parse})
 
+    def sentinel_masters(self):
+        "Returns a dictionary containing the master's state."
+        return self.execute_command('SENTINEL', 'masters',
+                                    parse='SENTINEL_INFO_MASTERS')
+
+    def sentinel_slaves(self, service_name):
+        "Returns a list of slaves for ``service_name``"
+        return self.execute_command('SENTINEL', 'slaves', service_name,
+                                    parse='SENTINEL_INFO')
+
+    def sentinel_sentinels(self, service_name):
+        "Returns a list of sentinels for ``service_name``"
+        return self.execute_command('SENTINEL', 'sentinels', service_name,
+                                    parse='SENTINEL_INFO')
+
+    def sentinel_get_master_addr_by_name(self, service_name):
+        "Returns a (host, port) pair for the given ``service_name``"
+        return self.execute_command('SENTINEL', 'get-master-addr-by-name',
+                                    service_name, parse='SENTINEL_ADDR_PORT')
+
     def shutdown(self):
         "Shutdown the server"
         try:
@@ -577,7 +664,9 @@ class StrictRedis(object):
     def delete(self, *names):
         "Delete one or more keys specified by ``names``"
         return self.execute_command('DEL', *names)
-    __delitem__ = delete
+
+    def __delitem__(self, name):
+        self.delete(name)
 
     def dump(self, name):
         """
@@ -806,7 +895,9 @@ class StrictRedis(object):
         if xx:
             pieces.append('XX')
         return self.execute_command('SET', *pieces)
-    __setitem__ = set
+
+    def __setitem__(self, name, value):
+        self.set(name, value)
 
     def setbit(self, name, offset, value):
         """
@@ -1086,6 +1177,71 @@ class StrictRedis(object):
         options = {'groups': len(get) if groups else None}
         return self.execute_command('SORT', *pieces, **options)
 
+    #### SCAN COMMANDS ####
+    def scan(self, cursor=0, match=None, count=None):
+        """
+        Scan and return (nextcursor, keys)
+
+        ``match`` allows for filtering the keys by pattern
+
+        ``count`` allows for hint the minimum number of returns
+        """
+        pieces = [cursor]
+        if match is not None:
+            pieces.extend(['MATCH', match])
+        if count is not None:
+            pieces.extend(['COUNT', count])
+        return self.execute_command('SCAN', *pieces)
+
+    def sscan(self, name, cursor=0, match=None, count=None):
+        """
+        Scan and return (nextcursor, members_of_set)
+
+        ``match`` allows for filtering the keys by pattern
+
+        ``count`` allows for hint the minimum number of returns
+        """
+        pieces = [name, cursor]
+        if match is not None:
+            pieces.extend(['MATCH', match])
+        if count is not None:
+            pieces.extend(['COUNT', count])
+        return self.execute_command('SSCAN', *pieces)
+
+    def hscan(self, name, cursor=0, match=None, count=None):
+        """
+        Scan and return (nextcursor, dict)
+
+        ``match`` allows for filtering the keys by pattern
+
+        ``count`` allows for hint the minimum number of returns
+        """
+        pieces = [name, cursor]
+        if match is not None:
+            pieces.extend(['MATCH', match])
+        if count is not None:
+            pieces.extend(['COUNT', count])
+        return self.execute_command('HSCAN', *pieces)
+
+    def zscan(self, name, cursor=0, match=None, count=None,
+              score_cast_func=float):
+        """
+        Scan and return (nextcursor, pairs)
+
+        ``match`` allows for filtering the keys by pattern
+
+        ``count`` allows for hint the minimum number of returns
+
+        ``score_cast_func`` a callable used to cast the score return value
+        """
+        pieces = [name, cursor]
+        if match is not None:
+            pieces.extend(['MATCH', match])
+        if count is not None:
+            pieces.extend(['COUNT', count])
+        options = {'score_cast_func': score_cast_func}
+        return self.execute_command('ZSCAN', *pieces, **options)
+
     #### SET COMMANDS ####
     def sadd(self, name, *values):
         "Add ``value(s)`` to set ``name``"
@@ -1153,7 +1309,7 @@ class StrictRedis(object):
         return self.execute_command('SREM', name, *values)
 
     def sunion(self, keys, *args):
-        "Return the union of sets specifiued by ``keys``"
+        "Return the union of sets specified by ``keys``"
         args = list_or_args(keys, args)
         return self.execute_command('SUNION', *args)
 
@@ -1289,20 +1445,20 @@ class StrictRedis(object):
         """
         return self.execute_command('ZREMRANGEBYSCORE', name, min, max)
 
-    def zrevrange(self, name, start, num, withscores=False,
+    def zrevrange(self, name, start, end, withscores=False,
                   score_cast_func=float):
         """
         Return a range of values from sorted set ``name`` between
-        ``start`` and ``num`` sorted in descending order.
+        ``start`` and ``end`` sorted in descending order.
 
-        ``start`` and ``num`` can be negative, indicating the end of the range.
+        ``start`` and ``end`` can be negative, indicating the end of the range.
 
         ``withscores`` indicates to return the scores along with the values
         The return type is a list of (value, score) pairs
 
         ``score_cast_func`` a callable used to cast the score return value
         """
-        pieces = ['ZREVRANGE', name, start, num]
+        pieces = ['ZREVRANGE', name, start, end]
         if withscores:
             pieces.append('withscores')
         options = {
@@ -1420,8 +1576,8 @@ class StrictRedis(object):
 
     def hmset(self, name, mapping):
         """
-        Sets each key in the ``mapping`` dict to its corresponding value
-        in the hash ``name``
+        Set key to value within hash ``name`` for each corresponding
+        key and value from the ``mapping`` dict.
         """
         if not mapping:
             raise DataError("'hmset' with 'mapping' of length 0")
@@ -1448,7 +1604,7 @@ class StrictRedis(object):
 
     def eval(self, script, numkeys, *keys_and_args):
         """
-        Execute the LUA ``script``, specifying the ``numkeys`` the script
+        Execute the Lua ``script``, specifying the ``numkeys`` the script
         will touch and the key names and argument values in ``keys_and_args``.
         Returns the result of the script.
 
@@ -1459,7 +1615,7 @@ class StrictRedis(object):
 
     def evalsha(self, sha, numkeys, *keys_and_args):
         """
-        Use the ``sha`` to execute a LUA script already registered via EVAL
+        Use the ``sha`` to execute a Lua script already registered via EVAL
         or SCRIPT LOAD. Specify the ``numkeys`` the script will touch and the
         key names and argument values in ``keys_and_args``. Returns the result
         of the script.
@@ -1484,21 +1640,21 @@ class StrictRedis(object):
         return self.execute_command('SCRIPT', 'FLUSH', **options)
 
     def script_kill(self):
-        "Kill the currently executing LUA script"
+        "Kill the currently executing Lua script"
         options = {'parse': 'KILL'}
         return self.execute_command('SCRIPT', 'KILL', **options)
 
     def script_load(self, script):
-        "Load a LUA ``script`` into the script cache. Returns the SHA."
+        "Load a Lua ``script`` into the script cache. Returns the SHA."
         options = {'parse': 'LOAD'}
         return self.execute_command('SCRIPT', 'LOAD', script, **options)
 
     def register_script(self, script):
         """
-        Register a LUA ``script`` specifying the ``keys`` it will touch.
+        Register a Lua ``script`` specifying the ``keys`` it will touch.
         Returns a Script object that is callable and hides the complexity of
         deal with scripts, keys, and shas. This is the preferred way to work
-        with LUA scripts.
+        with Lua scripts.
         """
         return Script(self, script)
 
@@ -1879,11 +2035,13 @@ class BasePipeline(object):
             errors.append((0, sys.exc_info()[1]))
 
         # and all the other commands
-        for i, _ in enumerate(commands):
+        for i, command in enumerate(commands):
             try:
                 self.parse_response(connection, '_')
             except ResponseError:
-                errors.append((i, sys.exc_info()[1]))
+                ex = sys.exc_info()[1]
+                self.annotate_exception(ex, i + 1, command[0])
+                errors.append((i, ex))
 
         # parse the EXEC.
         try:
@@ -1908,7 +2066,7 @@ class BasePipeline(object):
 
         # find any errors in the response and raise if necessary
         if raise_on_error:
-            self.raise_first_error(response)
+            self.raise_first_error(commands, response)
 
         # We have to run response callbacks manually
         data = []
@@ -1937,14 +2095,21 @@ class BasePipeline(object):
                 response.append(sys.exc_info()[1])
 
         if raise_on_error:
-            self.raise_first_error(response)
+            self.raise_first_error(commands, response)
         return response
 
-    def raise_first_error(self, response):
-        for r in response:
+    def raise_first_error(self, commands, response):
+        for i, r in enumerate(response):
             if isinstance(r, ResponseError):
+                self.annotate_exception(r, i + 1, commands[i][0])
                 raise r
 
+    def annotate_exception(self, exception, number, command):
+        cmd = unicode(' ').join(imap(unicode, command))
+        msg = unicode('Command # %d (%s) of pipeline caused error: %s') % (
+            number, cmd, unicode(exception.args[0]))
+        exception.args = (msg,) + exception.args[1:]
+
     def parse_response(self, connection, command_name, **options):
         result = StrictRedis.parse_response(
             self, connection, command_name, **options)
@@ -1967,9 +2132,11 @@ class BasePipeline(object):
 
     def execute(self, raise_on_error=True):
         "Execute all the commands in the current pipeline"
+        stack = self.command_stack
+        if not stack:
+            return []
         if self.scripts:
             self.load_scripts()
-        stack = self.command_stack
         if self.transaction or self.explicit_transaction:
             execute = self._execute_transaction
         else:
@@ -2027,7 +2194,7 @@ class Pipeline(BasePipeline, Redis):
 
 
 class Script(object):
-    "An executable LUA script object returned by ``register_script``"
+    "An executable Lua script object returned by ``register_script``"
 
     def __init__(self, registered_client, script):
         self.registered_client = registered_client
@@ -2138,6 +2305,7 @@ class Lock(object):
             raise ValueError("Cannot release an unlocked lock")
         existing = float(self.redis.get(self.name) or 1)
         # if the lock time is in the future, delete the lock
-        if existing >= self.acquired_until:
-            self.redis.delete(self.name)
+        delete_lock = existing >= self.acquired_until
         self.acquired_until = None
+        if delete_lock:
+            self.redis.delete(self.name)
